@@ -27,8 +27,9 @@ from .extractors import (discard_unwanted, extract_url_date,
                          DATESTRINGS_PATTERN, DATESTRINGS_CATCH,
                          SLASHES_PATTERN, SLASHES_YEAR,
                          YYYYMM_PATTERN, YYYYMM_CATCH, MMYYYY_PATTERN,
-                         MMYYYY_YEAR, SIMPLE_PATTERN)
-from .settings import HTML_CLEANER, MAX_POSSIBLE_CANDIDATES
+                         MMYYYY_YEAR, SIMPLE_PATTERN,
+                         THREE_COMP_REGEX_A, THREE_COMP_REGEX_B, TWO_COMP_REGEX)
+from .settings import CACHE_SIZE, HTML_CLEANER, MAX_POSSIBLE_CANDIDATES
 from .utils import load_html
 from .validators import (check_extracted_reference, compare_values,
                          convert_date, date_validator, filter_ymd_candidate,
@@ -41,28 +42,43 @@ def logstring(element):
     '''Format the element to be logged to a string.'''
     return html.tostring(element, pretty_print=False, encoding='unicode').strip()
 
-DATE_ATTRIBUTES = set([
+DATE_ATTRIBUTES = {
                   'article.created', 'article_date_original',
-                  'article.published', 'created', 'cxenseparse:recs:publishtime',
-                  'date', 'date_published', 'dc.date', 'dc.date.created',
-                  'dc.date.issued', 'dcterms.date', 'gentime', 'og:published_time',
+                  'article.published', 'article:published_time', 
+                  'bt:pubdate', 'citation_date', 'citation_publication_date',
+                  'created', 'cxenseparse:recs:publishtime',
+                  'date', 'date_published',
+                  'datecreated', 'dateposted', 'datepublished',
+                  # Dublin Core: https://wiki.whatwg.org/wiki/MetaExtensions
+                  'dc.date', 'dc.created', 'dc.date.created',
+                  'dc.date.issued', 'dc.date.publication',
+                  'dcterms.created', 'dcterms.date',
+                  'dcterms.issued', 'dc:created', 'dc:date',
+                  'gentime',
+                  # Open Graph: https://opengraphprotocol.org/
+                  'og:published_time', 'og:article:published_time',
                   'originalpublicationdate', 'parsely-pub-date',
                   'pubdate', 'publishdate', 'publish_date',
-                  'published-date', 'publication_date', 'sailthru.date',
-                  'timestamp',
-                  'article:published_time', 'bt:pubdate', 'datecreated',
-                  'dateposted', 'datepublished', 'dc:created', 'dc:date',
-                  'og:article:published_time', 'og:published_time',
-                  'sailthru.date', 'rnews:datepublished'
-                  ])
-PROPERTY_MODIFIED = set([
-                    'article:modified_time', 'datemodified', 'modified_time',
+                  'published-date', 'publication_date',  'rnews:datepublished',
+                  'sailthru.date', 'shareaholic:article_published_time', 'timestamp'
+                  }
+PROPERTY_MODIFIED = {
+                    'article:modified_time', 'datemodified',
+                    'dc.modified', 'dcterms.modified', 'modified_time',
                     'og:article:modified_time', 'og:updated_time',
                     'release_date', 'updated_time'
-                    ])
+                    }
+
+TRIM_REGEX = re.compile(r'[\n\r\s\t]+')
+NON_DIGITS_REGEX = re.compile(r'\D+$')
+GER_STRIP_REGEX = re.compile(r'^am ')
+
+LAST_MODIFIED = set(['lastmodified', 'last-modified'])
+ITEMPROP_ATTRS = set(['datecreated', 'datepublished', 'pubyear'])
+CLASS_ATTRS = set(['date-published', 'published', 'time published'])
 
 
-@lru_cache(maxsize=32)
+@lru_cache(maxsize=CACHE_SIZE)
 def examine_date_elements(tree, expression, outputformat, extensive_search, min_date, max_date):
     """Check HTML elements one by one for date expressions"""
     try:
@@ -76,12 +92,12 @@ def examine_date_elements(tree, expression, outputformat, extensive_search, min_
     attempt = None
     for elem in elements:
         # trim
-        textcontent = re.sub(r'[\n\r\s\t]+', ' ', elem.text_content(), re.MULTILINE).strip()
+        textcontent = TRIM_REGEX.sub(' ', elem.text_content(), re.MULTILINE).strip()
         # simple length heuristics
         if textcontent is not None and len(textcontent) > 6:
             # shorten and try the beginning of the string
             # trim non-digits at the end of the string
-            toexamine = re.sub(r'\D+$', '', textcontent[:48])
+            toexamine = NON_DIGITS_REGEX.sub('', textcontent[:48])
             # more than 4 digits required # list(filter(str.isdigit, toexamine))
             #if len([c for c in toexamine if c.isdigit()]) < 4:
             #    continue
@@ -95,7 +111,7 @@ def examine_date_elements(tree, expression, outputformat, extensive_search, min_
         # try link title (Blogspot)
         title_attr = elem.get('title')
         if title_attr is not None and len(title_attr) > 0:
-            toexamine = re.sub(r'\D+$', '', title_attr[:48])
+            toexamine = NON_DIGITS_REGEX.sub('', title_attr[:48])
             attempt = try_ymd_date(toexamine, outputformat, extensive_search, min_date, max_date)
             if attempt is not None:
                 break
@@ -140,20 +156,18 @@ def examine_header(tree, outputformat, extensive_search, original_date, min_date
         # property attribute
         if 'property' in elem.attrib and 'content' in elem.attrib:
             attribute = elem.get('property').lower()
-            # original date
-            if original_date is True:
-                if attribute in DATE_ATTRIBUTES:
-                    LOGGER.debug('examining meta property: %s', logstring(elem))
-                    headerdate = tryfunc(elem.get('content'))
-            # modified date: override published_time
-            else:
-                if attribute in PROPERTY_MODIFIED:
-                    LOGGER.debug('examining meta property: %s', logstring(elem))
-                    headerdate = tryfunc(elem.get('content'))
-                elif attribute in DATE_ATTRIBUTES:
-                    LOGGER.debug('examining meta property: %s', logstring(elem))
-                    headerdate = tryfunc(elem.get('content'))
-        # name attribute
+            # original date or modified date: override published_time
+            if (
+                original_date is True
+                and attribute in DATE_ATTRIBUTES
+                or original_date is not True
+                and (
+                    attribute in PROPERTY_MODIFIED
+                    or attribute in DATE_ATTRIBUTES
+                )
+            ):
+                LOGGER.debug('examining meta property: %s', logstring(elem))
+                headerdate = tryfunc(elem.get('content'))
         elif 'name' in elem.attrib and 'content' in elem.attrib:
             # url
             if elem.get('name').lower() == 'og:url':
@@ -163,7 +177,7 @@ def examine_header(tree, outputformat, extensive_search, original_date, min_date
                 LOGGER.debug('examining meta name: %s', logstring(elem))
                 headerdate = tryfunc(elem.get('content'))
             # modified
-            elif elem.get('name').lower() in ('lastmodified', 'last-modified'):
+            elif elem.get('name').lower() in LAST_MODIFIED:
                 LOGGER.debug('examining meta name: %s', logstring(elem))
                 if original_date is False:
                     headerdate = tryfunc(elem.get('content'))
@@ -173,19 +187,14 @@ def examine_header(tree, outputformat, extensive_search, original_date, min_date
             if elem.get('pubdate').lower() == 'pubdate':
                 LOGGER.debug('examining meta pubdate: %s', logstring(elem))
                 headerdate = tryfunc(elem.get('content'))
-        # other types # itemscope?
         elif 'itemprop' in elem.attrib:
             attribute = elem.get('itemprop').lower()
-            if attribute in (
-                    'datecreated', 'datepublished', 'pubyear'
-                ):
-                LOGGER.debug('examining meta itemprop: %s', logstring(elem))
-                if 'datetime' in elem.attrib:
-                    headerdate = try_ymd_date(elem.get('datetime'), outputformat, extensive_search, min_date, max_date)
-                elif 'content' in elem.attrib:
-                    headerdate = tryfunc(elem.get('content'))
-            # override
-            elif attribute == 'datemodified' and original_date is False:
+            # original: store / updated: override date
+            if (
+                attribute in ITEMPROP_ATTRS
+                or (attribute == 'datemodified'
+                and original_date is False)
+            ):
                 LOGGER.debug('examining meta itemprop: %s', logstring(elem))
                 if 'datetime' in elem.attrib:
                     headerdate = try_ymd_date(elem.get('datetime'), outputformat, extensive_search, min_date, max_date)
@@ -198,12 +207,15 @@ def examine_header(tree, outputformat, extensive_search, original_date, min_date
                     attempt = '-'.join([elem.get('content'), '01', '01'])
                     if date_validator(attempt, '%Y-%m-%d', latest=max_date) is True:
                         reserve = attempt
-        # http-equiv, rare http://www.standardista.com/html5/http-equiv-the-meta-attribute-explained/
+        # http-equiv, rare
         elif 'http-equiv' in elem.attrib:
             attribute = elem.get('http-equiv').lower()
             if attribute == 'date':
                 LOGGER.debug('examining meta http-equiv: %s', logstring(elem))
-                headerdate = tryfunc(elem.get('content'))
+                if original_date is True:
+                    headerdate = tryfunc(elem.get('content'))
+                else:
+                    reserve = tryfunc(elem.get('content'))
             elif attribute == 'last-modified':
                 LOGGER.debug('examining meta http-equiv: %s', logstring(elem))
                 if original_date is False:
@@ -274,7 +286,7 @@ def search_pattern(htmlstring, pattern, catch, yearpat, original_date, min_date,
 def try_expression(expression, outputformat, extensive_search, min_date, max_date):
     '''Check if the text string could be a valid date expression'''
     # trim
-    textcontent = re.sub(r'[\n\r\s\t]+', ' ', expression, re.MULTILINE).strip()
+    textcontent = TRIM_REGEX.sub(' ', expression, re.MULTILINE).strip()
     # simple length heuristics list(filter(str.isdigit, textcontent))
     if not textcontent or len([c for c in textcontent if c.isdigit()]) < 4:
         return None
@@ -282,15 +294,13 @@ def try_expression(expression, outputformat, extensive_search, min_date, max_dat
     return try_ymd_date(textcontent[:48], outputformat, extensive_search, min_date, max_date)
 
 
-@lru_cache(maxsize=32)
+@lru_cache(maxsize=CACHE_SIZE)
 def compare_reference(reference, expression, outputformat, extensive_search, original_date, min_date, max_date):
     '''Compare candidate to current date reference (includes date validation and older/newer test)'''
     attempt = try_expression(expression, outputformat, extensive_search, min_date, max_date)
     if attempt is not None:
-        new_reference = compare_values(reference, attempt, outputformat, original_date)
-    else:
-        new_reference = reference
-    return new_reference
+        return compare_values(reference, attempt, outputformat, original_date)
+    return reference
 
 
 def examine_abbr_elements(tree, outputformat, extensive_search, original_date, min_date, max_date):
@@ -307,37 +317,32 @@ def examine_abbr_elements(tree, outputformat, extensive_search, original_date, m
                     continue
                 LOGGER.debug('data-utime found: %s', candidate)
                 # look for original date
-                if original_date is True:
-                    if reference == 0:
-                        reference = candidate
-                    elif candidate < reference:
-                        reference = candidate
+                if original_date is True and (reference == 0 or candidate < reference):
+                    reference = candidate
                 # look for newest (i.e. largest time delta)
-                else:
-                    if candidate > reference:
-                        reference = candidate
+                elif original_date is False and candidate > reference:
+                    reference = candidate
             # class
-            if 'class' in elem.attrib:
-                if elem.get('class') in ('published', 'date-published', 'time published'):
-                    # other attributes
-                    if 'title' in elem.attrib:
-                        trytext = elem.get('title')
-                        LOGGER.debug('abbr published-title found: %s', trytext)
-                        # shortcut
-                        if original_date is True:
-                            attempt = try_ymd_date(trytext, outputformat, extensive_search, min_date, max_date)
-                            if attempt is not None:
-                                return attempt
-                        else:
-                            reference = compare_reference(reference, trytext, outputformat, extensive_search, original_date, min_date, max_date)
-                            # faster execution
-                            if reference > 0:
-                                break
-                    # dates, not times of the day
-                    if elem.text and len(elem.text) > 10:
-                        trytext = re.sub(r'^am ', '', elem.text)
-                        LOGGER.debug('abbr published found: %s', trytext)
+            if 'class' in elem.attrib and elem.get('class') in CLASS_ATTRS:
+                # other attributes
+                if 'title' in elem.attrib:
+                    trytext = elem.get('title')
+                    LOGGER.debug('abbr published-title found: %s', trytext)
+                    # shortcut
+                    if original_date is True:
+                        attempt = try_ymd_date(trytext, outputformat, extensive_search, min_date, max_date)
+                        if attempt is not None:
+                            return attempt
+                    else:
                         reference = compare_reference(reference, trytext, outputformat, extensive_search, original_date, min_date, max_date)
+                        # faster execution
+                        if reference > 0:
+                            break
+                # dates, not times of the day
+                if elem.text and len(elem.text) > 10:
+                    trytext = GER_STRIP_REGEX.sub('', elem.text)
+                    LOGGER.debug('abbr published found: %s', trytext)
+                    reference = compare_reference(reference, trytext, outputformat, extensive_search, original_date, min_date, max_date)
         # convert and return
         converted = check_extracted_reference(reference, outputformat, min_date, max_date)
         if converted is not None:
@@ -378,7 +383,7 @@ def examine_time_elements(tree, outputformat, extensive_search, original_date, m
                 else:
                     LOGGER.debug('time/datetime found: %s', elem.get('datetime'))
                 # analyze attribute
-                if shortcut_flag is True:
+                if shortcut_flag:
                     attempt = try_ymd_date(elem.get('datetime'), outputformat, extensive_search, min_date, max_date)
                     if attempt is not None:
                         return attempt
@@ -390,7 +395,7 @@ def examine_time_elements(tree, outputformat, extensive_search, original_date, m
             elif elem.text is not None and len(elem.text) > 6:
                 LOGGER.debug('time/datetime found: %s', elem.text)
                 reference = compare_reference(reference, elem.text, outputformat, extensive_search, original_date, min_date, max_date)
-            # else...
+            # else...?
         # return
         converted = check_extracted_reference(reference, outputformat, min_date, max_date)
         if converted is not None:
@@ -400,14 +405,12 @@ def examine_time_elements(tree, outputformat, extensive_search, original_date, m
 
 def normalize_match(match):
     '''Normalize string output by adding "0" if necessary.'''
-    if len(match.group(1)) == 1:
-        day = '0' + match.group(1)
-    else:
-        day = match.group(1)
-    if len(match.group(2)) == 1:
-        month = '0' + match.group(2)
-    else:
-        month = match.group(2)
+    day = match.group(1)
+    if len(day) == 1:
+        day = '0' + day
+    month = match.group(2)
+    if len(month) == 1:
+        month = '0' + month
     return day, month
 
 
@@ -458,9 +461,9 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     # YYYY-MM-DD/DD-MM-YYYY
     candidates = plausible_year_filter(htmlstring, SELECT_YMD_PATTERN, SELECT_YMD_YEAR)
     # revert DD-MM-YYYY patterns before sorting
-    replacement = dict()
+    replacement = {}
     for item in candidates:
-        match = re.match(r'([0-3]?[0-9])[/.-]([01]?[0-9])[/.-]([0-9]{4})', item)
+        match = THREE_COMP_REGEX_A.match(item)
         day, month = normalize_match(match)
         candidate = '-'.join([match.group(3), month, day])
         replacement[candidate] = candidates[item]
@@ -480,9 +483,9 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     # DD?/MM?/YY
     candidates = plausible_year_filter(htmlstring, SLASHES_PATTERN, SLASHES_YEAR, tocomplete=True)
     # revert DD-MM-YYYY patterns before sorting
-    replacement = dict()
+    replacement = {}
     for item in candidates:
-        match = re.match(r'([0-3]?[0-9])[/.]([01]?[0-9])[/.]([0-9]{2})', item)
+        match = THREE_COMP_REGEX_B.match(item)
         day, month = normalize_match(match)
         if match.group(3)[0] == '9':
             year = '19' + match.group(3)
@@ -502,21 +505,21 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     bestmatch = search_pattern(htmlstring, YYYYMM_PATTERN, YYYYMM_CATCH, YEAR_PATTERN, original_date, min_date, max_date)
     if bestmatch is not None:
         pagedate = '-'.join([bestmatch.group(1), bestmatch.group(2), '01'])
-        if date_validator(pagedate, '%Y-%m-%d', latest=max_date) is True:
-            if copyear == 0 or int(bestmatch.group(1)) >= copyear:
-                LOGGER.debug('date found for pattern "%s": %s', YYYYMM_PATTERN, pagedate)
-                return convert_date(pagedate, '%Y-%m-%d', outputformat)
+        if date_validator(pagedate, '%Y-%m-%d', latest=max_date) is True and (
+            copyear == 0 or int(bestmatch.group(1)) >= copyear
+        ):
+            LOGGER.debug('date found for pattern "%s": %s', YYYYMM_PATTERN, pagedate)
+            return convert_date(pagedate, '%Y-%m-%d', outputformat)
 
     # 2 components, second option
     candidates = plausible_year_filter(htmlstring, MMYYYY_PATTERN, MMYYYY_YEAR, original_date)
     # revert DD-MM-YYYY patterns before sorting
-    replacement = dict()
+    replacement = {}
     for item in candidates:
-        match = re.match(r'([0-3]?[0-9])[/.-]([0-9]{4})', item)
-        if len(match.group(1)) == 1:
-            month = '0' + match.group(1)
-        else:
-            month = match.group(1)
+        match = TWO_COMP_REGEX.match(item)
+        month = match.group(1)
+        if len(month) == 1:
+            month = '0' + month
         candidate = '-'.join([match.group(2), month, '01'])
         replacement[candidate] = candidates[item]
     candidates = Counter(replacement)
@@ -536,10 +539,11 @@ def search_page(htmlstring, outputformat, original_date, min_date, max_date):
     bestmatch = search_pattern(htmlstring, SIMPLE_PATTERN, YEAR_PATTERN, YEAR_PATTERN, original_date, min_date, max_date)
     if bestmatch is not None:
         pagedate = '-'.join([bestmatch.group(0), '01', '01'])
-        if date_validator(pagedate, '%Y-%m-%d', latest=max_date) is True:
-            if copyear == 0 or int(bestmatch.group(0)) >= copyear:
-                LOGGER.debug('date found for pattern "%s": %s', SIMPLE_PATTERN, pagedate)
-                return convert_date(pagedate, '%Y-%m-%d', outputformat)
+        if date_validator(pagedate, '%Y-%m-%d', latest=max_date) is True and (
+            copyear == 0 or int(bestmatch.group(0)) >= copyear
+        ):
+            LOGGER.debug('date found for pattern "%s": %s', SIMPLE_PATTERN, pagedate)
+            return convert_date(pagedate, '%Y-%m-%d', outputformat)
 
     return None
 
@@ -629,40 +633,34 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
     # expressions + text_content
     # first try in pruned tree
     search_tree, discarded = discard_unwanted(deepcopy(tree))
-    for expr in DATE_EXPRESSIONS:
-        dateresult = examine_date_elements(
-            search_tree, expr, outputformat, extensive_search, min_date, max_date
-        )
-        if dateresult is not None:
-            return dateresult
-    # search in discarded parts (currently: footer)
-    for subtree in discarded:
-        for expr in DATE_EXPRESSIONS:
-            dateresult = examine_date_elements(
-                subtree, expr, outputformat, extensive_search, min_date, max_date
-            )
-            if dateresult is not None:
-                return dateresult
-
+    dateresult = examine_date_elements(search_tree, DATE_EXPRESSIONS,
+        outputformat, extensive_search, min_date, max_date)
+    if dateresult is not None:
+        return dateresult
     # supply more expressions (other languages)
     if extensive_search is True:
-        for expr in ADDITIONAL_EXPRESSIONS:
-            dateresult = examine_date_elements(
-                tree, expr, outputformat, extensive_search, min_date, max_date
-            )
-            if dateresult is not None:
-                return dateresult
+        dateresult = examine_date_elements(search_tree, ADDITIONAL_EXPRESSIONS,
+            outputformat, extensive_search, min_date, max_date)
+        if dateresult is not None:
+            return dateresult
+
+    # search in discarded parts (currently: footers and archive.org-banner)
+    for subtree in discarded:
+        dateresult = examine_date_elements(subtree, DATE_EXPRESSIONS,
+            outputformat, extensive_search, min_date, max_date)
+        if dateresult is not None:
+            return dateresult
 
     # try time elements
     time_result = examine_time_elements(
-        tree, outputformat, extensive_search, original_date, min_date, max_date
+        search_tree, outputformat, extensive_search, original_date, min_date, max_date
     )
     if time_result is not None:
         return time_result
 
     # clean before string search
     try:
-        cleaned_html = HTML_CLEANER.clean_html(tree)
+        cleaned_html = HTML_CLEANER.clean_html(deepcopy(tree))
     # rare LXML error: no NULL bytes or control characters
     except ValueError:
         cleaned_html = tree
@@ -685,7 +683,7 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
         return text_result
 
     # title
-    for title_elem in cleaned_html.iterfind('.//title|.//h1'):
+    for title_elem in tree.iter('title', 'h1'):
         attempt = try_ymd_date(title_elem.text_content(), outputformat, extensive_search, min_date, max_date)
         if attempt is not None:
             return attempt
