@@ -21,12 +21,6 @@ EXTERNAL_PARSER = dateparser.DateDataParser(settings={
     'DATE_ORDER': 'DMY',
 })
 
-# potential regex speedup
-#try:
-import regex
-#except ImportError:
-#    regex = re
-
 # allow_redetect_language=False, languages=['de', 'en'],
 #EXTERNAL_PARSER_CONFIG = {
 #    'PREFER_DAY_OF_MONTH': 'first', 'PREFER_DATES_FROM': 'past',
@@ -48,17 +42,20 @@ except ImportError:
     parse_datetime = parse_datetime_as_naive = FULL_PARSE  # shortcut
 
 # own
+from .settings import CACHE_SIZE
 from .validators import convert_date, date_validator
 
 LOGGER = logging.getLogger(__name__)
 
-DATE_EXPRESSIONS = [
-    """.//*[contains(@id, 'date') or contains(@id, 'Date') or
+DATE_EXPRESSIONS = """
+    .//*[contains(@id, 'date') or contains(@id, 'Date') or
     contains(@id, 'datum') or contains(@id, 'Datum') or contains(@id, 'time')
-    or contains(@class, 'post-meta-time')]""",
-    """.//*[contains(@class, 'date') or contains(@class, 'Date')
-    or contains(@class, 'datum') or contains(@class, 'Datum')]""",
-    """.//*[contains(@class, 'postmeta') or contains(@class, 'post-meta')
+    or contains(@class, 'post-meta-time')]
+    |
+    .//*[contains(@class, 'date') or contains(@class, 'Date')
+    or contains(@class, 'datum') or contains(@class, 'Datum')]
+    |
+    .//*[contains(@class, 'postmeta') or contains(@class, 'post-meta')
     or contains(@class, 'entry-meta') or contains(@class, 'entry-date') or contains(@class, 'postMeta')
     or contains(@class, 'post_meta') or contains(@class, 'post__meta') or
     contains(@class, 'article__date') or contains(@class, 'post_detail') or @class='meta'
@@ -68,91 +65,109 @@ DATE_EXPRESSIONS = [
     contains(@class, 'dateline') or contains(@class, 'subline')
     or contains(@class, 'published') or contains(@class, 'posted') or
     contains(@class, 'submitted') or contains(@class, 'updated') or contains(@class, 'created-post')
-    or contains(@id, 'post-timestamp') or contains(@class, 'post-timestamp')]""",
-    """.//*[contains(@id, 'lastmod') or contains(@itemprop, 'date') or
-    contains(@class, 'time') or contains(@id, 'metadata') or contains(@id, 'publish')]""",
-    ".//footer|.//*[@class='post-footer' or @class='footer' or @id='footer']",
-    ".//small",
-    """.//*[contains(@class, 'author') or contains(@class, 'autor') or
+    or contains(@id, 'post-timestamp') or contains(@class, 'post-timestamp')]
+    |
+    .//*[contains(@id, 'lastmod') or contains(@itemprop, 'date') or
+    contains(@class, 'time') or contains(@id, 'metadata') or contains(@id, 'publish')]
+    |
+    .//footer
+    |
+    .//*[@class='post-footer' or @class='footer' or @id='footer']
+    |
+    .//small
+    |
+    .//*[contains(@class, 'author') or contains(@class, 'autor') or
     contains(@class, 'field-content') or @class='meta' or
     contains(@class, 'info') or contains(@class, 'fa-clock-o') or contains(@class, 'fa-calendar') or
-    contains(@class, 'publication')]""",
-]
+    contains(@class, 'publication')]"""
 
 # supply more expressions for more languages
-ADDITIONAL_EXPRESSIONS = [
-    ".//*[contains(@class, 'fecha') or contains(@class, 'parution')]",
-]
+ADDITIONAL_EXPRESSIONS = ".//*[contains(@class, 'fecha') or contains(@class, 'parution')]"
 
 # discard parts of the webpage
-DISCARD_EXPRESSIONS = [
-    './/footer',
-    './/*[(self::div or self::section)][@id="footer" or @class="footer"]',
-]
+DISCARD_EXPRESSIONS = """.//footer
+    |.//*[(self::div or self::section)][@id="footer" or @class="footer"]
+    |.//div[@id="wm-ipp-base" or @id="wm-ipp"]"""  # archive.org banner inserts
 
 # Regex cache
-MDY_PATTERN = re.compile(r'''(January|February|March|April|May|June|July|
-August|September|October|November|December|Jan|Feb|Mar|Apr|Jun|Jul|Aug|Sep|
-Oct|Nov|Dec|Januar|Jänner|Februar|Feber|März|Mai|Juni|Juli|Oktober|Dezember|
+YMD_NO_SEP_PATTERN = re.compile(r'(?:\D|^)(\d{8})(?:\D|$)')
+YMD_PATTERN = re.compile(r'(?:\D|^)(\d{4})[\-/.](\d{1,2})[\-/.](\d{1,2})(?:\D|$)')
+DMY_PATTERN = re.compile(r'(?:\D|^)(\d{1,2})[\-/.](\d{1,2})[\-/.](\d{2,4})(?:\D|$)')
+YM_PATTERN = re.compile(r'(?:\D|^)(\d{4})[\-/.](\d{1,2})(?:\D|$)')
+MY_PATTERN = re.compile(r'(?:\D|^)(\d{1,2})[\-/.](\d{4})(?:\D|$)')
+LONG_MDY_PATTERN = re.compile(r'''(
+January|February|March|April|May|June|July|August|September|October|November|December|
+Januari|Februari|Maret|Mei|Juni|Juli|Agustus|Oktober|Desember|
+Jan|Feb|Mar|Apr|Jun|Jul|Aug|Sep|Oct|Nov|Dec|
+Januar|Jänner|Februar|Feber|März|Mai|Dezember|
+janvier|février|mars|avril|mai|juin|juillet|aout|septembre|octobre|novembre|décembre|
 Ocak|Şubat|Mart|Nisan|Mayıs|Haziran|Temmuz|Ağustos|Eylül|Ekim|Kasım|Aralık|
-Oca|Şub|Mar|Nis|Haz|Tem|Ağu|Eyl|Eki|Kas|
-Ara) ([0-9]{1,2})(st|nd|rd|th)?,? ([0-9]{4})'''.replace('\n', ''))
-DMY_PATTERN = re.compile(r'''([0-9]{1,2})(st|nd|rd|th)? (of )?(January|
-February|March|April|May|June|July|August|September|October|November|December|
-Jan|Feb|Mar|Apr|Jun|Jul|Aug|Sep|Oct|Nov|Dec|Januar|Jänner|Februar|Feber|März|
-Mai|Juni|Juli|Oktober|Dezember|Ocak|Şubat|Mart|Nisan|Mayıs|Haziran|Temmuz|
-Ağustos|Eylül|Ekim|Kasım|Aralık|Oca|Şub|Mar|Nis|Haz|Tem|Ağu|Eyl|Eki|Kas|
-Ara),? ([0-9]{4})'''.replace('\n', ''))
-ENGLISH_DATE = re.compile(r'([0-9]{1,2})/([0-9]{1,2})/([0-9]{2,4})')
+Oca|Şub|Mar|Nis|Haz|Tem|Ağu|Eyl|Eki|Kas|Ara
+) ([0-9]{1,2})(st|nd|rd|th)?,? ([0-9]{4})'''.replace('\n', ''))
+LONG_DMY_PATTERN = re.compile(r'''([0-9]{1,2})(st|nd|rd|th)? (of )?(
+January|February|March|April|May|June|July|August|September|October|November|December|
+Januari|Februari|Maret|Mei|Juni|Juli|Agustus|Oktober|Desember|
+Jan|Feb|Mar|Apr|Jun|Jul|Aug|Sep|Oct|Nov|Dec|
+Januar|Jänner|Februar|Feber|März|Mai|Dezember|
+janvier|février|mars|avril|mai|juin|juillet|aout|septembre|octobre|novembre|décembre|
+Ocak|Şubat|Mart|Nisan|Mayıs|Haziran|Temmuz|Ağustos|Eylül|Ekim|Kasım|Aralık|
+Oca|Şub|Mar|Nis|Haz|Tem|Ağu|Eyl|Eki|Kas|Ara
+),? ([0-9]{4})'''.replace('\n', ''))
 COMPLETE_URL = re.compile(r'([0-9]{4})[/-]([0-9]{1,2})[/-]([0-9]{1,2})')
 PARTIAL_URL = re.compile(r'/([0-9]{4})/([0-9]{1,2})/')
-YMD_PATTERN = re.compile(r'([0-9]{4})-([0-9]{2})-([0-9]{2})')
-DATESTUB_PATTERN = re.compile(r'([0-9]{1,2})\.([0-9]{1,2})\.([0-9]{2,4})')
 GERMAN_TEXTSEARCH = re.compile(r'''([0-9]{1,2})\.? (Januar|Jänner|Februar|Feber|März|April|
 Mai|Juni|Juli|August|September|Oktober|November|Dezember) ([0-9]{4})'''.replace('\n', ''))
-GENERAL_TEXTSEARCH = re.compile(r'''January|February|March|April|May|June|July|
-August|September|October|November|December|Jan|Feb|Mar|Apr|Jun|Jul|Aug|Sep|Oct|
-Nov|Dec|Januar|Jänner|Februar|Feber|März|Mai|Juni|Juli|Oktober|Dezember|
-Ocak|Şubat|Mart|Nisan|Mayıs|Haziran|Temmuz|Ağustos|Eylül|Ekim|Kasım|Aralık|
-Oca|Şub|Mar|Nis|Haz|Tem|Ağu|Eyl|Eki|Kas|Ara'''.replace('\n', ''))
 JSON_PATTERN_MODIFIED = \
   re.compile(r'"dateModified": ?"([0-9]{4}-[0-9]{2}-[0-9]{2})')
 JSON_PATTERN_PUBLISHED = \
   re.compile(r'"datePublished": ?"([0-9]{4}-[0-9]{2}-[0-9]{2})')
-TIMESTAMP_PATTERN = regex.compile(r'([0-9]{4}-[0-9]{2}-[0-9]{2}|[0-9]{2}\.[0-9]{2}\.[0-9]{4}).[0-9]{2}:[0-9]{2}:[0-9]{2}')
+TIMESTAMP_PATTERN = re.compile(r'([0-9]{4}-[0-9]{2}-[0-9]{2}|[0-9]{2}\.[0-9]{2}\.[0-9]{4}).[0-9]{2}:[0-9]{2}:[0-9]{2}')
 
-# English + German + Turkish dates cache
-TEXT_MONTHS = {'Januar': '01', 'Jänner': '01', 'January': '01', 'Jan': '01',
-               'Ocak': '01', 'Oca': '01',
-               'Februar': '02', 'Feber': '02', 'February': '02', 'Feb': '02',
-               'Şubat': '02', 'Şub': '02',
-               'März': '03', 'March': '03', 'Mar': '03', 'Mart': '03',
-               'April': '04', 'Apr': '04', 'Nisan': '04', 'Nis': '04',
-               'Mai': '05', 'May': '05', 'Mayıs': '05',
-               'Juni': '06', 'June': '06', 'Jun': '06',
-               'Haziran': '06', 'Haz': '06',
-               'Juli': '07', 'July': '07', 'Jul': '07',
-               'Temmuz': '07', 'Tem': '07',
-               'August': '08', 'Aug': '08',
-               'Ağustos': '08', 'Ağu': '08',
-               'September': '09', 'Sep': '09',
-               'Eylül': '09', 'Eyl': '09',
-               'Oktober': '10', 'October': '10', 'Oct': '10',
-               'Ekim': '10', 'Eki': '10',
-               'November': '11', 'Nov': '11',
-               'Kasım': '11', 'Kas': '11',
-               'Dezember': '12', 'December': '12', 'Dec': '12',
-               'Aralık': '12', 'Ara': '12'}
+# English, French, German, Indonesian and Turkish dates cache
+TEXT_MONTHS = {
+    # January
+    'Januar': '01', 'Jänner': '01', 'January': '01', 'Januari': '01', 'Jan': '01',
+    'Ocak': '01', 'Oca': '01', 'janvier': '01',
+    # February
+    'Februar': '02', 'Feber': '02', 'February': '02', 'Februari': '02', 'Feb': '02',
+    'Şubat': '02', 'Şub': '02', 'février': '02',
+    # March
+    'März': '03', 'March': '03', 'Maret': '03', 'Mar': '03', 'Mart': '03', 'mars': '03',
+    # April
+    'April': '04', 'Apr': '04', 'Nisan': '04', 'Nis': '04', 'avril': '04',
+    # May
+    'Mai': '05', 'May': '05', 'Mei': '05', 'Mayıs': '05', 'mai': '05',
+    # June
+    'Juni': '06', 'June': '06', 'Jun': '06', 'Haziran': '06', 'Haz': '06', 'juin': '06',
+    # July
+    'Juli': '07', 'July': '07', 'Jul': '07', 'Temmuz': '07', 'Tem': '07', 'juillet': '07',
+    # August
+    'August': '08', 'Agustus': '08', 'Aug': '08', 'Ağustos': '08', 'Ağu': '08', 'aout': '08',
+    # September
+    'September': '09', 'Sep': '09', 'Eylül': '09', 'Eyl': '09', 'septembre': '09',
+    # October
+    'Oktober': '10', 'October': '10', 'Oct': '10', 'Ekim': '10', 'Eki': '10', 'octobre': '10',
+    # November
+    'November': '11', 'Nov': '11', 'Kasım': '11', 'Kas': '11', 'novembre': '11',
+    # December
+    'Dezember': '12', 'December': '12', 'Desember': '12', 'Dec': '12', 'Aralık': '12',
+    'Ara': '12', 'décembre': '12'
+}
 
 TEXT_DATE_PATTERN = re.compile(r'[.:,_/ -]|^[0-9]+$')
-NO_TEXT_DATE_PATTERN = re.compile(r'[0-9]{3,}\D+[0-9]{3,}|[0-9]{2}:[0-9]{2}(:| )|\D*[0-9]{4}\D*$')
+NO_TEXT_DATE_PATTERN = re.compile(r'\D+[0-9]{3,}\D+|[0-9]{3,}\D+[0-9]{3,}|[0-9]{2}:[0-9]{2}(:| )|\D*[0-9]{4}\D*$|\+[0-9]{2}\D+')
 
 # use of regex module for speed
-EN_PATTERNS = regex.compile(r'(?:[Dd]ate[^0-9"]{,20}|updated|published) *?(?:in)? *?:? *?([0-9]{1,4})[./]([0-9]{1,2})[./]([0-9]{2,4})', re.I)
-DE_PATTERNS = regex.compile(r'(?:Datum|Stand): ?([0-9]{1,2})\.([0-9]{1,2})\.([0-9]{2,4})')
-TR_PATTERNS = regex.compile(r'''(?:güncellen?me|yayı(?:m|n)lan?ma) *?(?:tarihi)? *?:? *?([0-9]{1,2})[./]([0-9]{1,2})[./]([0-9]{2,4})|
+EN_PATTERNS = re.compile(r'(?:[Dd]ate[^0-9"]{,20}|updated|published) *?(?:in)? *?:? *?([0-9]{1,4})[./]([0-9]{1,2})[./]([0-9]{2,4})', re.I)
+DE_PATTERNS = re.compile(r'(?:Datum|Stand): ?([0-9]{1,2})\.([0-9]{1,2})\.([0-9]{2,4})')
+TR_PATTERNS = re.compile(r'''(?:güncellen?me|yayı(?:m|n)lan?ma) *?(?:tarihi)? *?:? *?([0-9]{1,2})[./]([0-9]{1,2})[./]([0-9]{2,4})|
 ([0-9]{1,2})[./]([0-9]{1,2})[./]([0-9]{2,4}) *?(?:'de|'da|'te|'ta|’de|’da|’te|’ta|tarihinde) *(?:güncellendi|yayı(?:m|n)landı)'''.replace('\n', ''), re.I)
 
+# core patterns
+THREE_COMP_REGEX_A = re.compile(r'([0-3]?[0-9])[/.-]([01]?[0-9])[/.-]([0-9]{4})')
+THREE_COMP_REGEX_B = re.compile(r'([0-3]?[0-9])[/.-]([01]?[0-9])[/.-]([0-9]{2})')
+TWO_COMP_REGEX = re.compile(r'([0-3]?[0-9])[/.-]([0-9]{4})')
+
 # extensive search patterns
 YEAR_PATTERN = re.compile(r'^\D?(199[0-9]|20[0-9]{2})')
 COPYRIGHT_PATTERN = re.compile(r'(?:©|\&copy;|Copyright|\(c\))\D*(?:[12][0-9]{3}-)?([12][0-9]{3})\D')
@@ -177,10 +192,9 @@ SIMPLE_PATTERN = re.compile(r'\D(199[0-9]|20[0-9]{2})\D')
 def discard_unwanted(tree):
     '''Delete unwanted sections of an HTML document and return them as a list'''
     my_discarded = []
-    for expr in DISCARD_EXPRESSIONS:
-        for subtree in tree.xpath(expr):
-            my_discarded.append(subtree)
-            subtree.getparent().remove(subtree)
+    for subtree in tree.xpath(DISCARD_EXPRESSIONS):
+        my_discarded.append(subtree)
+        subtree.getparent().remove(subtree)
     return tree, my_discarded
 
 
@@ -218,9 +232,15 @@ def extract_partial_url_date(testurl, outputformat):
     return None
 
 
+def correct_year(year):
+    """Adapt year from YY to YYYY format"""
+    if year < 100:
+        year += 1900 if year >= 90 else 2000
+    return year
+
+
 def regex_parse(string):
     """Full-text parse using a series of regular expressions"""
-    dateobject = None
     dateobject = regex_parse_de(string)
     if dateobject is None:
         dateobject = regex_parse_multilingual(string)
@@ -249,42 +269,37 @@ def regex_parse_de(string):
 def regex_parse_multilingual(string):
     """Try full-text parse for English date elements"""
     # https://github.com/vi3k6i5/flashtext ?
-    # numbers
-    match = ENGLISH_DATE.search(string)
+
+    # American English
+    match = LONG_MDY_PATTERN.search(string)
     if match:
-        day, month, year = match.group(2), match.group(1), match.group(3)
+        day, month, year = match.group(2), TEXT_MONTHS[match.group(1)], match.group(4)
+    # multilingual day-month-year pattern
     else:
-        # general search
-        if not GENERAL_TEXTSEARCH.search(string):
-            return None
-        # American English
-        match = MDY_PATTERN.search(string)
+        match = LONG_DMY_PATTERN.search(string)
         if match:
-            day, month, year = match.group(2), TEXT_MONTHS[match.group(1)], \
-                               match.group(4)
-        # multilingual day-month-year pattern
+            day, month, year = match.group(1), TEXT_MONTHS[match.group(4)], match.group(5)
         else:
-            match = DMY_PATTERN.search(string)
-            if match:
-                day, month, year = match.group(1), TEXT_MONTHS[match.group(4)], \
-                                   match.group(5)
-            else:
-                return None
+            return None
+
     # process and return
-    if len(year) == 2:
-        year = '20' + year
     try:
-        dateobject = datetime.date(int(year), int(month), int(day))
+        int_day, int_month, int_year = int(day), int(month), int(year)
+        int_year = correct_year(int_year)
+        dateobject = datetime.date(int_year, int_month, int_day)
     except ValueError:
         return None
-    LOGGER.debug('English text parse: %s', dateobject)
+
+    LOGGER.debug('multilingual text found: %s', dateobject)
     return dateobject
 
 
+# TODO: check what's necessary here and what's not
 def custom_parse(string, outputformat, extensive_search, min_date, max_date):
     """Try to bypass the slow dateparser"""
     LOGGER.debug('custom parse test: %s', string)
-    # '201709011234' not covered by dateparser # regex was too slow
+
+    # 1. '201709011234' not covered by dateparser, and regex too slow
     if string[0:8].isdigit():
         try:
             candidate = datetime.date(int(string[:4]),
@@ -295,7 +310,8 @@ def custom_parse(string, outputformat, extensive_search, min_date, max_date):
         if date_validator(candidate, '%Y-%m-%d') is True:
             LOGGER.debug('ymd match: %s', candidate)
             return convert_date(candidate, '%Y-%m-%d', outputformat)
-    # much faster
+
+    # 2. shortcut, much faster
     if string[0:4].isdigit():
         # try speedup with ciso8601 (if installed)
         try:
@@ -309,51 +325,72 @@ def custom_parse(string, outputformat, extensive_search, min_date, max_date):
                 return result.strftime(outputformat)
         except (OverflowError, TypeError, ValueError):
             LOGGER.debug('parsing error: %s', string)
-    # %Y-%m-%d search
+
+    # 3. Try YYYYMMDD, use regex
+    match = YMD_NO_SEP_PATTERN.search(string)
+    if match:
+        try:
+            year, month, day = int(match.group(0)[:4]), int(match.group(0)[4:6]), int(match.group(0)[6:8])
+            candidate = datetime.date(year, month, day)
+        except ValueError:
+            LOGGER.debug('YYYYMMDD value error: %s', match.group(0))
+        else:
+            if date_validator(candidate, '%Y-%m-%d') is True:
+                LOGGER.debug('YYYYMMDD match: %s', candidate)
+                return convert_date(candidate, '%Y-%m-%d', outputformat)
+
+    # 4. Try Y-M-D pattern since it's the one used in ISO-8601
     match = YMD_PATTERN.search(string)
     if match:
         try:
-            candidate = datetime.date(int(match.group(1)),
-                                      int(match.group(2)),
-                                      int(match.group(3)))
+            day, month, year = int(match.group(3)), int(match.group(2)), int(match.group(1))
+            candidate = datetime.date(year, month, day)
         except ValueError:
-            LOGGER.debug('value error: %s', match.group(0))
+            LOGGER.debug('Y-M-D value error: %s', match.group(0))
         else:
             if date_validator(candidate, '%Y-%m-%d') is True:
-                LOGGER.debug('ymd match: %s', candidate)
+                LOGGER.debug('Y-M-D match: %s', candidate)
                 return convert_date(candidate, '%Y-%m-%d', outputformat)
-    # faster than fire dateparser at once
-    datestub = DATESTUB_PATTERN.search(string)
-    if datestub and len(datestub.group(3)) in (2, 4):
+
+    # 5. Try the D-M-Y pattern since it's the most common date format in the world
+    match = DMY_PATTERN.search(string)
+    if match:
         try:
-            if len(datestub.group(3)) == 2:
-                candidate = datetime.date(int('20' + datestub.group(3)),
-                                          int(datestub.group(2)),
-                                          int(datestub.group(1)))
-            elif len(datestub.group(3)) == 4:
-                candidate = datetime.date(int(datestub.group(3)),
-                                          int(datestub.group(2)),
-                                          int(datestub.group(1)))
+            day, month, year = int(match.group(1)), int(match.group(2)), int(match.group(3))
+            year = correct_year(year)
+            # If month is more than 12, swap it with the day
+            if month > 12 and day <= 12:
+                day, month = month, day
+            candidate = datetime.date(year, month, day)
         except ValueError:
-            LOGGER.debug('value error: %s', datestub.group(0))
+            LOGGER.debug('D-M-Y value error: %s', match.group(0))
         else:
-            # test candidate
             if date_validator(candidate, '%Y-%m-%d') is True:
-                LOGGER.debug('D.M.Y match: %s', candidate)
+                LOGGER.debug('D-M-Y match: %s', candidate)
                 return convert_date(candidate, '%Y-%m-%d', outputformat)
-    # text match
+
+    # 6. Try the Y-M pattern
+    match = YM_PATTERN.search(string)
+    if match:
+        try:
+            year, month = int(match.group(1)), int(match.group(2))
+            candidate = datetime.date(year, month, 1)
+        except ValueError:
+            LOGGER.debug('Y-M value error: %s', match.group(0))
+        else:
+            if date_validator(candidate, '%Y-%m-%d') is True:
+                LOGGER.debug('Y-M match: %s', candidate)
+                return convert_date(candidate, '%Y-%m-%d', outputformat)
+
+    # 7. Try the other regex pattern
     dateobject = regex_parse(string)
-    # copyright match?
-    #if dateobject is None:
-    # © Janssen-Cilag GmbH 2014-2019. https://www.krebsratgeber.de/artikel/was-macht-eine-zelle-zur-krebszelle
-    # examine
-    if dateobject is not None:
+    if date_validator(dateobject, outputformat) is True:
         try:
-            if date_validator(dateobject, outputformat) is True:
-                LOGGER.debug('custom parse result: %s', dateobject)
-                return dateobject.strftime(outputformat)
+            LOGGER.debug('custom parse result: %s', dateobject)
+            return dateobject.strftime(outputformat)
         except ValueError as err:
             LOGGER.debug('value error during conversion: %s %s', string, err)
+
     return None
 
 
@@ -375,30 +412,35 @@ def external_date_parser(string, outputformat):
     return None
 
 
-@lru_cache(maxsize=32)
+@lru_cache(maxsize=CACHE_SIZE)
 def try_ymd_date(string, outputformat, extensive_search, min_date, max_date):
     """Use a series of heuristics and rules to parse a potential date expression"""
-    # discard on formal criteria
-    # list(filter(str.isdigit, string))
+    # if string less than 6 chars, stop
     if not string or len(string) < 6:
         return None
-    digits_num = len([c for c in string if c.isdigit()])
-    if not 4 <= digits_num <= 18:
+
+    # count how many digit number in this string
+    if not 4 <= len([c for c in string if c.isdigit()]) <= 18:
         return None
-    # just time/single year or digits, not a date
+
+    # check if string only contains time/single year or digits and not a date
     if not TEXT_DATE_PATTERN.search(string) or NO_TEXT_DATE_PATTERN.match(string):
         return None
-    # faster
+
+    # try to parse using the faster method
     customresult = custom_parse(string, outputformat, extensive_search, min_date, max_date)
     if customresult is not None:
         return customresult
-    # slow but extensive search
+
+    # use slow but extensive search
     if extensive_search is True:
         # send to date parser
         dateparser_result = external_date_parser(string, outputformat)
-        if dateparser_result is not None:
-            if date_validator(dateparser_result, outputformat, earliest=min_date, latest=max_date):
-                return dateparser_result
+        if date_validator(
+            dateparser_result, outputformat, earliest=min_date, latest=max_date
+        ):
+            return dateparser_result
+
     return None
 
 
@@ -407,7 +449,7 @@ def img_search(tree, outputformat, min_date, max_date):
     element = tree.find('.//meta[@property="og:image"]')
     if element is not None and 'content' in element.attrib:
         result = extract_url_date(element.get('content'), outputformat)
-        if result is not None and date_validator(result, outputformat, earliest=min_date, latest=max_date) is True:
+        if date_validator(result, outputformat, earliest=min_date, latest=max_date) is True:
             return result
     return None
 
@@ -421,7 +463,7 @@ def json_search(tree, outputformat, original_date, min_date, max_date):
         json_pattern = JSON_PATTERN_MODIFIED
     # look throughout the HTML tree
     for elem in tree.xpath('.//script[@type="application/ld+json"]|//script[@type="application/settings+json"]'):
-        if not elem.text or not '"date' in elem.text:
+        if not elem.text or '"date' not in elem.text:
             continue
         json_match = json_pattern.search(elem.text)
         if json_match and date_validator(json_match.group(1), '%Y-%m-%d', earliest=min_date, latest=max_date):
@@ -454,10 +496,9 @@ def extract_idiosyncrasy(idiosyncrasy, htmlstring, outputformat, min_date, max_d
                                       int(match.group(groups[2])),
                                       int(match.group(groups[3])))
         elif len(match.group(groups[3])) in (2, 4):
-            # switch to MM/DD/YY
+            # swap variables to switch to MM/DD/YY
             if int(match.group(groups[2])) > 12:
-                tmp1, tmp2 = groups[1], groups[2]
-                groups[1], groups[2] = tmp2, tmp1
+                groups[1], groups[2] = groups[2], groups[1]
             # DD/MM/YY
             try:
                 if len(match.group(groups[3])) == 2:
@@ -470,10 +511,9 @@ def extract_idiosyncrasy(idiosyncrasy, htmlstring, outputformat, min_date, max_d
                                               int(match.group(groups[1])))
             except ValueError:
                 LOGGER.debug('value error in idiosyncrasies: %s', match.group(0))
-    if candidate is not None:
-        if date_validator(candidate, '%Y-%m-%d', earliest=min_date, latest=max_date) is True:
-            LOGGER.debug('idiosyncratic pattern found: %s', match.group(0))
-            return convert_date(candidate, '%Y-%m-%d', outputformat)
+    if date_validator(candidate, '%Y-%m-%d', earliest=min_date, latest=max_date) is True:
+        LOGGER.debug('idiosyncratic pattern found: %s', match.group(0))
+        return convert_date(candidate, '%Y-%m-%d', outputformat)
     return None
 
 
