@@ -5,6 +5,7 @@ Unit tests for the htmldate library.
 
 
 import datetime
+import io
 import logging
 import os
 import re
@@ -13,6 +14,7 @@ import sys
 import pytest
 
 from collections import Counter
+from contextlib import redirect_stdout
 from unittest.mock import patch
 
 try:
@@ -24,16 +26,11 @@ except ImportError:
 
 from lxml import html
 
-try:
-    import cchardet as chardet
-except ImportError:
-    import chardet
-
-from htmldate.cli import examine, parse_args
+from htmldate.cli import examine, parse_args, process_args
 from htmldate.core import compare_reference, find_date, search_page, search_pattern, select_candidate, try_ymd_date
 from htmldate.extractors import custom_parse, external_date_parser, extract_partial_url_date, regex_parse
 from htmldate.settings import MIN_DATE, MIN_DATE, LATEST_POSSIBLE
-from htmldate.utils import fetch_url, load_html
+from htmldate.utils import decode_response, detect_encoding, fetch_url, load_html
 from htmldate.validators import convert_date, date_validator, get_max_date, get_min_date, output_format_validator
 
 
@@ -110,35 +107,23 @@ MOCK_PAGES = {
 'https://www.acredis.com/schoenheitsoperationen/augenlidstraffung/': 'acredis.com.augenlidstraffung.html',
 'https://www.hertie-school.org/en/debate/detail/content/whats-on-the-cards-for-von-der-leyen/': 'hertie-school.org.leyen.html',
 'https://www.adac.de/rund-ums-fahrzeug/tests/kindersicherheit/kindersitztest-2018/': 'adac.de.kindersitztest.html',
+'http://web.archive.org/web/20210916140120/https://www.kath.ch/die-insel-der-klosterzoeglinge/': 'archive.org.kath.ch.html',
+}
+
+MEDIACLOUD_PAGES = {
+    'pagename' : 'thing.html'
 }
-# '': '', \
 
 
-def load_mock_page(url):
+def load_mock_page(url, dirname='cache', dictname=MOCK_PAGES):
     '''load mock page from samples'''
-    try:
-        with open(os.path.join(TEST_DIR, 'cache', MOCK_PAGES[url]), 'r') as inputf:
-            htmlstring = inputf.read()
-    # encoding/windows fix for the tests
-    except UnicodeDecodeError:
-        # read as binary
-        with open(os.path.join(TEST_DIR, 'cache', MOCK_PAGES[url]), 'rb') as inputf:
-            htmlbinary = inputf.read()
-        guessed_encoding = chardet.detect(htmlbinary)['encoding']
-        if guessed_encoding is not None:
-            try:
-                htmlstring = htmlbinary.decode(guessed_encoding)
-            except UnicodeDecodeError:
-                htmlstring = htmlbinary
-        else:
-            print('Encoding error')
-    return htmlstring
+    with open(os.path.join(TEST_DIR, dirname, dictname[url]), 'rb') as inputf:
+        return inputf.read()
 
 
-# def new_pages():
-#    '''New pages, to be sorted'''
-    # assert find_date(load_mock_page('...')) == 'YYYY-MM-DD'
-#    pass
+def load_mediacloud_page(url):
+    '''load mediacloud page from samples'''
+    return load_mock_page(url, dirname='test_set', dictname=MEDIACLOUD_PAGES)
 
 
 def test_input():
@@ -148,7 +133,11 @@ def test_input():
     assert load_html('<'*100) is None
     assert load_html(b'<html><body>XYZ</body></html>') is not None
     assert load_html(b'<'*100) is None
+    assert load_html(b'<html><body>\x2f\x2e\x9f</body></html>') is not None
+    assert load_html('<html><body>\x2f\x2e\x9f</body></html>'.encode('latin-1')) is not None
     assert load_html('https://httpbin.org/html') is not None
+    assert decode_response(b'\x1f\x8babcdef') is not None
+    # find_date logic
     assert find_date(None) is None
     # min and max date output
     assert get_min_date('2020-02-20') == datetime.date(2020, 2, 20)
@@ -174,7 +163,6 @@ def test_exact_date():
     ## HTML tree
     assert find_date('<html><head><meta property="dc:created" content="2017-09-01"/></head><body></body></html>') == '2017-09-01'
     assert find_date('<html><head><meta property="og:published_time" content="2017-09-01"/></head><body></body></html>', original_date=True) == '2017-09-01'
-    assert find_date('<html><head><meta http-equiv="date" content="2017-09-01"/></head><body></body></html>', original_date=True) == '2017-09-01'
     assert find_date('<html><head><meta name="last-modified" content="2017-09-01"/></head><body></body></html>', original_date=False) == '2017-09-01'
     assert find_date('<html><head><meta property="OG:Updated_Time" content="2017-09-01"/></head><body></body></html>', extensive_search=False) == '2017-09-01'
     assert find_date('<html><head><Meta Property="og:updated_time" content="2017-09-01"/></head><body></body></html>', extensive_search=False) == '2017-09-01'
@@ -195,10 +183,16 @@ def test_exact_date():
     assert find_date('<html><head><meta itemprop="datecreated" datetime="2018-02-02"/></head><body></body></html>') == '2018-02-02'
     assert find_date('<html><head><meta itemprop="datemodified" content="2018-02-04"/></head><body></body></html>') == '2018-02-04'
     assert find_date('<html><head><meta http-equiv="last-modified" content="2018-02-05"/></head><body></body></html>') == '2018-02-05'
+    assert find_date('<html><head><meta http-equiv="date" content="2017-09-01"/></head><body></body></html>', original_date=True) == '2017-09-01'
+    assert find_date('<html><head><meta http-equiv="last-modified" content="2018-10-01"/><meta http-equiv="date" content="2017-09-01"/></head><body></body></html>', original_date=True) == '2017-09-01'
+    assert find_date('<html><head><meta http-equiv="last-modified" content="2018-10-01"/><meta http-equiv="date" content="2017-09-01"/></head><body></body></html>', original_date=False) == '2018-10-01'
+    assert find_date('<html><head><meta http-equiv="date" content="2017-09-01"/><meta http-equiv="last-modified" content="2018-10-01"/></head><body></body></html>', original_date=True) == '2017-09-01'
+    assert find_date('<html><head><meta http-equiv="date" content="2017-09-01"/><meta http-equiv="last-modified" content="2018-10-01"/></head><body></body></html>', original_date=False) == '2018-10-01'
     assert find_date('<html><head><meta name="Publish_Date" content="02.02.2004"/></head><body></body></html>') == '2004-02-02'
     assert find_date('<html><head><meta name="pubDate" content="2018-02-06"/></head><body></body></html>') == '2018-02-06'
     assert find_date('<html><head><meta pubdate="pubDate" content="2018-02-06"/></head><body></body></html>') == '2018-02-06'
     assert find_date('<html><head><meta itemprop="DateModified" datetime="2018-02-06"/></head><body></body></html>') == '2018-02-06'
+    assert find_date('<html><head><meta name="DC.Issued" content="2021-07-13"/></head><body></body></html>') == '2021-07-13'
 
     ## time in document body
     assert find_date(load_mock_page('https://www.facebook.com/visitaustria/'), original_date=True) == '2017-10-06'
@@ -207,9 +201,13 @@ def test_exact_date():
     assert find_date('<html><body><time datetime="08:00"></body></html>') is None
     assert find_date('<html><body><time datetime="2014-07-10 08:30:45.687"></body></html>') == '2014-07-10'
     assert find_date('<html><head></head><body><time class="entry-time" itemprop="datePublished" datetime="2018-04-18T09:57:38+00:00"></body></html>') == '2018-04-18'
-    print(find_date('<html><body><footer class="article-footer"><p class="byline">Veröffentlicht am <time class="updated" datetime="2019-01-03T14:56:51+00:00">3. Januar 2019 um 14:56 Uhr.</time></p></footer></body></html>') == '2019-01-03')
     assert find_date('<html><body><footer class="article-footer"><p class="byline">Veröffentlicht am <time class="updated" datetime="2019-01-03T14:56:51+00:00">3. Januar 2019 um 14:56 Uhr.</time></p></footer></body></html>') == '2019-01-03'
     assert find_date('<html><body><footer class="article-footer"><p class="byline">Veröffentlicht am <time class="updated" datetime="2019-01-03T14:56:51+00:00"></time></p></footer></body></html>') == '2019-01-03'
+    assert find_date('<html><body><time datetime="2011-09-27" class="entry-date"></time><time datetime="2011-09-28" class="updated"></time></body></html>', original_date=True) == '2011-09-27'
+    # problem here:
+    #assert find_date('<html><body><time datetime="2011-09-27" class="entry-date"></time><time datetime="2011-09-28" class="updated"></time></body></html>', original_date=False) == '2011-09-28'
+    #assert find_date('<html><body><time datetime="2011-09-28" class="updated"></time><time datetime="2011-09-27" class="entry-date"></time></body></html>', original_date=True) == '2011-09-27'
+    assert find_date('<html><body><time datetime="2011-09-28" class="updated"></time><time datetime="2011-09-27" class="entry-date"></time></body></html>', original_date=False) == '2011-09-28'
     # removed from HTML5 https://www.w3schools.com/TAgs/att_time_datetime_pubdate.asp
     assert find_date('<html><body><time datetime="2011-09-28" pubdate="pubdate"></time></body></html>', original_date=False) == '2011-09-28'
     assert find_date('<html><body><time datetime="2011-09-28" pubdate="pubdate"></time></body></html>', original_date=True) == '2011-09-28'
@@ -285,13 +283,13 @@ def test_exact_date():
     assert find_date(load_mock_page('http://carta.info/der-neue-trend-muss-statt-wunschkoalition/')) == '2012-05-08'
     assert find_date(load_mock_page('https://www.wunderweib.de/manuela-reimann-hochzeitsueberraschung-in-bayern-107930.html')) == '2019-06-20'
     assert find_date(load_mock_page('https://www.befifty.de/home/2017/7/12/unter-uns-montauk')) == '2017-07-12'
-    assert find_date(load_mock_page('https://www.brigitte.de/aktuell/riverdale--so-ehrt-die-serie-luke-perry-in-staffel-vier-11602344.html')) == '2019-06-20'
+    assert find_date(load_mock_page('https://www.brigitte.de/aktuell/riverdale--so-ehrt-die-serie-luke-perry-in-staffel-vier-11602344.html')) == '2019-06-20' #returns an error
     assert find_date(load_mock_page('http://www.loldf.org/spip.php?article717')) == '2019-06-27'
     assert find_date(load_mock_page('https://www.beltz.de/sachbuch_ratgeber/buecher/produkt_produktdetails/37219-12_wege_zu_guter_pflege.html')) == '2019-02-07'
     assert find_date(load_mock_page('https://www.oberstdorf-resort.de/interaktiv/blog/unser-kraeutergarten-wannenkopfhuette.html')) == '2018-06-20'
     assert find_date(load_mock_page('https://www.wienbadminton.at/news/119843/Come-Together')) == '2018-05-06'
     assert find_date(load_mock_page('https://www.ldt.de/ldtblog/fall-in-love-with-black/')) == '2017-08-08'
-    assert find_date(load_mock_page('https://paris-luttes.info/quand-on-comprend-que-les-grenades-12355'), original_date=True) == '2019-07-03' # should be '2019-06-29'
+    assert find_date(load_mock_page('https://paris-luttes.info/quand-on-comprend-que-les-grenades-12355'), original_date=True) == '2019-06-29'
     assert find_date(load_mock_page('https://verfassungsblog.de/the-first-decade/')) == '2019-07-13'
     assert find_date(load_mock_page('https://cric-grenoble.info/infos-locales/article/putsh-en-cours-a-radio-kaleidoscope-1145')) == '2019-06-09'
     assert find_date(load_mock_page('https://www.sebastian-kurz.at/magazin/wasserstoff-als-schluesseltechnologie')) == '2019-07-30'
@@ -323,6 +321,11 @@ def test_exact_date():
     assert find_date('<html><body><a class="ribbon date " title="12th December 2018" href="https://example.org/" itemprop="url">Text</a></body></html>') == '2018-12-12'
 
 
+    # archive.org documents
+    assert find_date(load_mock_page('http://web.archive.org/web/20210916140120/https://www.kath.ch/die-insel-der-klosterzoeglinge/'), extensive_search=False) is None
+    assert find_date(load_mock_page('http://web.archive.org/web/20210916140120/https://www.kath.ch/die-insel-der-klosterzoeglinge/'), extensive_search=True) == '2021-07-13'
+
+
 def test_approximate_date():
     '''this page should return an approximate date'''
     # copyright text
@@ -330,7 +333,6 @@ def test_approximate_date():
     # other
     assert find_date(load_mock_page('https://creativecommons.org/about/'), original_date=False) == '2017-08-11' # or '2017-08-03'
     assert find_date(load_mock_page('https://creativecommons.org/about/'), original_date=True) == '2016-05-22' # or '2017-08-03'
-    # problem on Windows
     assert find_date(load_mock_page('https://www.deutschland.de/en')) == '2017-08-01'
     assert find_date(load_mock_page('http://www.greenpeace.org/international/en/campaigns/forests/asia-pacific/')) == '2017-04-28'
     assert find_date(load_mock_page('https://www.creativecommons.at/faircoin-hackathon')) == '2017-07-24'
@@ -371,8 +373,6 @@ def test_output_format_validator():
     assert output_format_validator('%M-%Y') is True
     assert output_format_validator('ABC') is False
     assert output_format_validator(123) is False
-    # problem with some Python versions: AssertionError: assert True is False
-    # assert output_format_validator('X%') is False
 
 
 def test_try_ymd_date():
@@ -386,7 +386,7 @@ def test_try_ymd_date():
     assert try_ymd_date('Am 1. September 2017 um 15:36 Uhr schrieb', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) == '2017-09-01'
     assert try_ymd_date('Fri - September 1 - 2017', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) == '2017-09-01'
     assert try_ymd_date('1.9.2017', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) == '2017-09-01'
-    assert try_ymd_date('1/9/17', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) == '2017-01-09' # assuming MDY format
+    assert try_ymd_date('1/9/17', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) == '2017-09-01' # assuming MDY format
     assert try_ymd_date('201709011234', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) == '2017-09-01'
     # other output format
     assert try_ymd_date('1.9.2017', '%d %B %Y', True, MIN_DATE, LATEST_POSSIBLE) == '01 September 2017'
@@ -435,9 +435,13 @@ def test_regex_parse():
     assert regex_parse('3rd Tuesday in March') is None
     assert regex_parse('Mart 26, 2019') is not None
     assert regex_parse('Salı, Mart 26, 2019') is not None
-    assert regex_parse('3/14/2016') is not None
     assert regex_parse('36/14/2016') is None
+    assert regex_parse('January 36 1998') is None
+    assert custom_parse('January 12 1098', OUTPUTFORMAT, False, MIN_DATE, LATEST_POSSIBLE) is None
+    assert custom_parse('1998-01', OUTPUTFORMAT, False, MIN_DATE, LATEST_POSSIBLE) is not None
+    assert custom_parse('10.10.98', OUTPUTFORMAT, False, MIN_DATE, LATEST_POSSIBLE) is not None
     assert custom_parse('12122004', OUTPUTFORMAT, False, MIN_DATE, LATEST_POSSIBLE) is None
+    assert custom_parse('3/14/2016', OUTPUTFORMAT, False, MIN_DATE, LATEST_POSSIBLE) is not None
     assert custom_parse('20041212', OUTPUTFORMAT, False, MIN_DATE, LATEST_POSSIBLE) is not None
     assert custom_parse('20041212', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) is not None
     assert custom_parse('1212-20-04', OUTPUTFORMAT, False, MIN_DATE, LATEST_POSSIBLE) is None
@@ -624,7 +628,9 @@ def test_search_pattern(original_date=False, min_date=MIN_DATE, max_date=LATEST_
 def test_search_html(original_date=False, min_date=MIN_DATE, max_date=LATEST_POSSIBLE):
     '''test pattern search in HTML'''
     # file input + output format
-    assert search_page(load_mock_page('http://www.heimicke.de/chronik/zahlen-und-daten/'), '%d %B %Y', original_date, min_date, max_date) == '06 April 2019'
+    fileinput = load_mock_page('http://www.heimicke.de/chronik/zahlen-und-daten/')
+    encoding = detect_encoding(fileinput)
+    assert search_page(fileinput.decode(encoding), '%d %B %Y', original_date, min_date, max_date) == '06 April 2019'
     # tree input
     assert search_page('<html><body><p>The date is 5/2010</p></body></html>', OUTPUTFORMAT, original_date, min_date, max_date) == '2010-05-01'
     assert search_page('<html><body><p>The date is 5.5.2010</p></body></html>', OUTPUTFORMAT, original_date, min_date, max_date) == '2010-05-05'
@@ -683,6 +689,30 @@ def test_cli():
     assert examine('<html><body>2016-07-12</body></html>', extensive_bool=True, maxdate='2015-01-01') is None
     assert examine('<html><body>2016-07-12</body></html>', extensive_bool=True, maxdate='2017-12-31') == '2016-07-12'
     assert examine('<html><body>2016-07-12</body></html>', extensive_bool=True, maxdate='2017-41-41') == '2016-07-12'
+    # first test
+    testargs = ['', '-u', 'https://httpbin.org/html']
+    with patch.object(sys, 'argv', testargs):
+        args = parse_args(testargs)
+    f = io.StringIO()
+    with redirect_stdout(f):
+        process_args(args)
+    assert len(f.getvalue()) == 0
+    # second test
+    args.URL = None
+    args.inputfile = os.path.join(TEST_DIR, 'testlist.txt')
+    f = io.StringIO()
+    with redirect_stdout(f):
+        process_args(args)
+    assert f.getvalue() == 'https://httpbin.org/html\tNone\n'
+    # third test: Linux and MacOS only
+    if os.name != 'nt':
+        args.inputfile = None
+        sys.stdin = open(os.path.join(TEST_DIR, 'cache', 'befifty.montauk.html'), 'r')
+        f = io.StringIO()
+        with redirect_stdout(f):
+            process_args(args)
+        assert f.getvalue() == '2017-07-12\n'
+        sys.stdin = sys.__stdin__
 
 
 def test_download():
